unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, FileCtrl, ExtCtrls;

type
  TMainForm = class(TForm)
    GbxCopy: TGroupBox;
    CopyDriveBox: TDriveComboBox;
    BtnCopy: TButton;
    BtnPlay: TButton;
    TitleImage: TImage;
    GbxOptions: TGroupBox;
    ChkFullscreenMovie: TCheckBox;
    ProgressLabel: TLabel;
    procedure BtnCopyClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ChkFullscreenMovieClick(Sender: TObject);
    procedure BtnPlayClick(Sender: TObject);
  private
    procedure CopyProgress(fil: string);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  IniFiles, ShellAPI;

procedure TMainForm.CopyProgress(fil: string);
resourcestring
  LNG_COPY_PROGRESS = 'Copying %s ...';
begin
  if fil = '' then
  begin
    ProgressLabel.Visible := false;
  end
  else
  begin
    ProgressLabel.Visible := true;
    ProgressLabel.Caption := Format(LNG_COPY_PROGRESS, [fil]);
  end;
  Application.ProcessMessages;
end;

function BaseDir: string;
begin
  result := ExtractFilePath(ParamStr(0));
end;

procedure UpdateIniDrive(drive: char);
var
  ini: TIniFile;
begin
  ini := TIniFile.Create(BaseDir+'Config.ini');
  try
    ini.WriteString('PATHS', 'MOVIES',  BaseDir+'Movies\');
    ini.WriteString('PATHS', 'MUSIC',   BaseDir+'Music\');
    ini.WriteString('PATHS', 'WAVES',   BaseDir+'Audio\');
    ini.WriteString('PATHS', 'ArtFile', BaseDir+'blownawy.art');
    ini.WriteString('PATHS', 'DLL',     BaseDir+'Animdll.dll');
  finally
    ini.Free;
  end;
end;

function RemoveReadOnly(fil: string): boolean;
begin
  result := FileSetAttr(fil, FileGetAttr(fil) and not faReadOnly) = 0;
end;

procedure TMainForm.BtnCopyClick(Sender: TObject);

  function _(subdir: string): boolean;
  var
    SR: TSearchRec;
  begin
    result := true;

    if FileExists(CopyDriveBox.Drive+':\'+subdir) then
    begin
      CopyProgress(subdir);
      RemoveReadOnly(BaseDir+subdir);
      if not CopyFile(PChar(CopyDriveBox.Drive+':\'+subdir),
                      PChar(BaseDir+subdir),
                      false) then
      begin
        Result := false;
        Exit;
      end;
    end
    else if FindFirst(CopyDriveBox.Drive+':\'+subdir+'\*.*', faArchive, SR) = 0 then
    begin
      repeat
        if not DirectoryExists(BaseDir+subdir) then MkDir(BaseDir+subdir);
        CopyProgress(subdir+'\'+SR.Name);
        RemoveReadOnly(BaseDir+subdir+'\'+SR.Name);
        if not CopyFile(PChar(CopyDriveBox.Drive+':\'+subdir+'\'+SR.Name),
                        PChar(BaseDir+subdir+'\'+SR.Name),
                        false) then
        begin
          Result := false;
          Exit;
        end;
      until FindNext(SR) <> 0;
      FindClose(SR);
    end
    else
    begin
      Result := false;
      Exit;
    end;
  end;

resourcestring
  LNG_CDROM_FAIL = 'Could not find Blown Away CD-ROM in drive %s:\';
  LNG_COPY_DONE = 'All files are now copied to your hard disk. You can now play the game without the CD-ROM.';
  LNG_COPY_ERROR = 'Error while copying the files';
begin
  if not DirectoryExists(CopyDriveBox.Drive+':\Audio') or
     not DirectoryExists(CopyDriveBox.Drive+':\Movies') or
     not DirectoryExists(CopyDriveBox.Drive+':\Music') or
     not FileExists(CopyDriveBox.Drive+':\blownawy.art') then
  begin
    MessageDlg(Format(LNG_CDROM_FAIL, [UpperCase(CopyDriveBox.Drive)]), mtError, [mbOk], 0);
    Exit;
  end;

  BtnCopy.Enabled := false;
  BtnPlay.Enabled := false;
  CopyDriveBox.Enabled := false;
  try
    if not _('Audio')  or
       not _('Movies') or
       not _('Music')  or
       not _('blownawy.art') or
       not _('Animdll.dll') or
       not _('blownawy.exe') then
    begin
      MessageDlg(LNG_COPY_ERROR, mtError, [mbOk], 0);
    end
    else
    begin
      UpdateIniDrive(CopyDriveBox.Drive);
      MessageDlg(LNG_COPY_DONE, mtInformation, [mbOk], 0);
    end;
  finally
    BtnCopy.Enabled := true;
    BtnPlay.Enabled := true;
    CopyDriveBox.Enabled := true;
    CopyProgress('');
  end;
end;

procedure TMainForm.FormShow(Sender: TObject);
resourcestring
  LNG_NOT_FOUND = '%s not found.';
var
  ini: TIniFile;
  s: string;
begin
  if not FileExists(BaseDir+'Config.ini') then
  begin
    MessageDlg(Format(LNG_NOT_FOUND, ['Config.ini']), mtError, [mbOk], 0);
    Close;
    Exit;
  end;
  if not FileExists(BaseDir+'Blownawy.exe') then
  begin
    MessageDlg(Format(LNG_NOT_FOUND, ['Blownawy.exe']), mtError, [mbOk], 0);
    Close;
    Exit;
  end;

  ini := TIniFile.Create(BaseDir+'Config.ini');
  try
    s := ini.ReadString('PATHS', 'ArtFile', BaseDir);
    if Length(s) = 0 then exit;
    CopyDriveBox.Drive := s[1];

    ChkFullscreenMovie.Checked := ini.ReadBool('Movies', 'FullScreen', true);
  finally
    ini.Free;
  end;
end;

procedure TMainForm.ChkFullscreenMovieClick(Sender: TObject);
var
  ini: TIniFile;
begin
  ini := TIniFile.Create(BaseDir+'Config.ini');
  try
    ini.WriteBool('Movies', 'FullScreen', ChkFullscreenMovie.Checked);
  finally
    ini.Free;
  end;
end;

procedure TMainForm.BtnPlayClick(Sender: TObject);
begin
  ShellExecute(Handle, 'open', 'Blownawy.exe', '', PChar(BaseDir), SW_NORMAL);
  Close;
end;

end.
